/*
 * jQuery UI Monthpicker
 *
 * @licensed MIT <see below>
 * @licensed GPL <see below>
 *
 * @author Luciano Costa
 * http://lucianocosta.info/jquery.mtz.monthpicker/
 *
 * Depends:
 *  jquery.ui.core.js
 */

/**
 * MIT License
 * Copyright (c) 2011, Luciano Costa
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
/**
 * GPL License
 * Copyright (c) 2011, Luciano Costa
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

 /*!
	* Extended CSS-Classes, HTML-Tags by Jaroslav Herber
	* modified method: mountWidget
  */

;(function ($) {

	var methods = {
		init : function (options) {
			return this.each(function () {
				var
					$this = $(this),
					data = $this.data('monthpicker'),
					year = (options && options.year) ? options.year : (new Date()).getFullYear(),
					month = (options && options.month) ? options.month : (new Date()).getMonth()+1,
					settings = $.extend({
						pattern: 'mm/yyyy',
						selectedMonth: month,
						selectedMonthName: '',
						selectedYear: year,
						startYear: year - 10,
						finalYear: year + 10,
						monthNames: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
						id: "monthpicker_" + (Math.random() * Math.random()).toString().replace('.', ''),
						openOnFocus: true,
						disabledMonths: []
					}, options);

				settings.dateSeparator = settings.pattern.replace(/(mmm|mm|m|yyyy|yy|y)/ig,'');

				// If the plugin hasn't been initialized yet for this element
				if (!data) {
					$(this).data('monthpicker', {
						'target': $this,
						'settings': settings
					});

					if (settings.openOnFocus === true) {
						$this.bind('focus', function () {
							$this.monthpicker('show');
						});
					}

					$this.monthpicker('mountWidget', settings);
					$this.bind('monthpicker-click-month', function (e, month, year) {
						$this.monthpicker('setValue', settings);
						$this.monthpicker('hide');
					});

					// hide widget when user clicks elsewhere on page
					$this.addClass("mtz-monthpicker-widgetcontainer");
					$(document).unbind("mousedown.mtzmonthpicker").bind("mousedown.mtzmonthpicker", function (e) {
						var target = $(e.target);
						if ( !(target.is('.jh-monthpicker') || target.is('.jh-monthpicker *')) || target.is('.ui-datepicker-close') ) {
							$(".mtz-monthpicker-widgetcontainer").each(function () {
								if (typeof($(this).data("monthpicker"))!="undefined") {
									$(this).monthpicker('hide');
								}
							});
						}
					});
				}

			});
		},

		show: function (n) {
			var widget = $('#' + this.data('monthpicker').settings.id);
			var monthpicker = $('#' + this.data('monthpicker').target.attr("id") + ':eq(0)');
			widget.css("top", monthpicker.offset().top  + monthpicker.outerHeight());
			widget.css("left", monthpicker.offset().left  );
			widget.fadeIn('fast');
			//widget.find('select').focus();
			this.trigger('monthpicker-show');
		},
		
		getDate: function () {
			var month = $(this).data('monthpicker').settings.selectedMonth;
			var year = $(this).data('monthpicker').settings.selectedYear;
			var ret = new Date(year,month-1,1,0,0,0,0);
			return ret;
		},
		
		setDate: function (date) {
			var
				$this = $(this), i = 0,
				monthpicker = this,
				settings = this.data('monthpicker').settings,
				container = $('#' + settings.id);
			
			settings.selectedYear = date.getFullYear();
			settings.selectedMonth = date.getMonth()+1;

			// Set input-field
			$this.monthpicker('setValue', settings);
				
			// Set month
			container.find('.ui-datepicker-current-day').removeClass('ui-datepicker-current-day');
			container.find('.mtz-monthpicker-month').each(function () {
				var m = settings.selectedMonth;
				if ($.inArray(m, settings.disabledMonths) < 0 && i == m ) {
					$(this).parent('td').addClass('ui-datepicker-current-day');
					monthpicker.trigger('monthpicker-click-month', m);
				}
				i++;
			});
				
			// Set year
			container.find('.mtz-monthpicker-year').val(settings.selectedYear);
			monthpicker.trigger('monthpicker-change-year', settings.selectedYear);
			
			// Show layer
			//$this.monthpicker('show');
		},

		hide: function () {
			var widget = $('#' + this.data('monthpicker').settings.id);
			if (widget.is(':visible')) {
				widget.fadeOut('fast');
				this.trigger('monthpicker-hide');
			}
		},

		setValue: function (settings) {
			var
				month = settings.selectedMonth,
				year = settings.selectedYear;
				
			if(settings.pattern.indexOf('mmm') >= 0) {
				month = settings.selectedMonthName;
			} else if(settings.pattern.indexOf('mm') >= 0 && settings.selectedMonth < 10) {
				month = '0' + settings.selectedMonth;
			}

			if(settings.pattern.indexOf('yyyy') < 0) {
				year = year.toString().substr(2,2);
			}

			if (settings.pattern.indexOf('y') > settings.pattern.indexOf(settings.dateSeparator)) {
				this.val(month + settings.dateSeparator + year);
			} else {
				this.val(year + settings.dateSeparator + month);
			}

			this.change();
		},

		disableMonths: function (months) {
			var
				settings = this.data('monthpicker').settings,
				container = $('#' + settings.id);

			settings.disabledMonths = months;

			container.find('.mtz-monthpicker-month').each(function () {
				var m = parseInt($(this).data('month'));
				if ($.inArray(m, months) >= 0) {
					$(this).addClass('ui-state-disabled');
				} else {
					$(this).removeClass('ui-state-disabled');
				}
			});
		},

		mountWidget: function (settings) {
			var
				monthpicker = this,
				container = $('<div id="'+ settings.id +'" class="ui-datepicker jh-monthpicker" />'),
				header = $('<div class="mtz-monthpicker jh-monthpicker-header ui-datepicker-header" />'),
				selectContainer = $('<div class="jh-monthpicker-select-container" />'),
				combo = $('<select class="mtz-monthpicker mtz-monthpicker-year jh-monthpicker-select" />'),
				table = $('<table class="ui-datepicker-calendar" />'),
				tbody = $('<tbody />'),
				tr = $('<tr />'),
				td = '',
				selectedYear = settings.selectedYear,
				selectedMonth = settings.selectedMonth,
				option = null,
				attrSelectedYear = $(this).data('selected-year'),
				attrStartYear = $(this).data('start-year'),
				attrFinalYear = $(this).data('final-year');

			if( settings.pattern.match(/^mm/) ) {
				r = monthpicker.val().match(/^(0[1-9]|1[0-2])/);
				if(r) {
					r = parseInt(r[0]);
					settings.selectedMonth = r;
				}
			}

			if (attrStartYear) {
				settings.startYear = attrStartYear;
			}

			if (attrFinalYear) {
				settings.finalYear = attrFinalYear;
			}
			
			if (attrSelectedYear) {
				settings.selectedYear = attrSelectedYear;
			} else if(settings.pattern.match(/yyyy$/) ) {
				r = monthpicker.val().match(/(19|20)\d{2}$/);
				if(r) {
					r = parseInt(r[0]);
					if( r >= settings.startYear && r <= settings.finalYear ) {
						settings.selectedYear = r;
					}
				}
			}

			container.css({
				position:'absolute',
				zIndex:999999,
				whiteSpace:'nowrap',
				width:'250px',
				overflow:'hidden',
				textAlign:'center',
				display:'none',
				top: monthpicker.offset().top + monthpicker.outerHeight(),
				left: monthpicker.offset().left
			});

			// mount years combo
			for (var i = settings.startYear; i <= settings.finalYear; i++) {
				var option = $('<option class="mtz-monthpicker" />').attr('value', i).append(i);
				if (settings.selectedYear === i) {
					option.attr('selected', 'selected');
				}
				combo.append(option);
			}

			header.append(selectContainer);
			selectContainer.append(combo);
			header.appendTo(container);

			// mount months table
			for (var i=1; i<=12; i++) {
				td = $('<td />');
				if (settings.selectedMonth === i) {
					td.addClass('ui-datepicker-current-day');
				}
				
				td.append('<a href="#" class="ui-state-default mtz-monthpicker mtz-monthpicker-month month" data-month="'+i+'">'+settings.monthNames[i-1]+'</a>');
				tr.append(td).appendTo(tbody);
				if (i % 3 === 0) {
					tr = $('<tr />');
				}
			}

			table.append(tbody).appendTo(container);

			container.find('.mtz-monthpicker-month').bind('click', function () {
				container.find('.ui-datepicker-current-day').removeClass('ui-datepicker-current-day');
				var m = parseInt($(this).data('month'));
				if ($.inArray(m, settings.disabledMonths) < 0 ) {
					$(this).parent('td').addClass('ui-datepicker-current-day');
					settings.selectedMonth = m;
					settings.selectedMonthName = $(this).text();
					monthpicker.trigger('monthpicker-click-month', m);
				}
				return false;
			});

			container.find('.mtz-monthpicker-year').bind('change', function () {
				settings.selectedYear = $(this).val();
				monthpicker.trigger('monthpicker-change-year', $(this).val());
			});

			container.append('<div class="ui-datepicker-buttonpane ui-widget-content"><button data-event="click" data-handler="hide" class="ui-datepicker-close ui-state-default ui-priority-primary ui-corner-all" type="button"></button></div>');
			container.appendTo('body');
		},

		destroy: function () {
			return this.each(function () {
				// TODO: look for other things to remove
				$(this).removeData('monthpicker');
			});
		}

	};

	$.fn.monthpicker = function (method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call( arguments, 1 ));
		} else if (typeof method === 'object' || ! method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error('Method ' + method + ' does not exist on jQuery.mtz.monthpicker');
		}
	};

})(jQuery);